package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.core.prediction.entity.PlantPredictedPowerMidTermEnt;
import pps.core.prediction.entity.PlantPredictedPowerShortTermEnt;
import pps.core.prediction.mapper.PlantPredictedPowerMidTermMapper;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermMapper;
import pps.core.prediction.service.data.predicted_api.PredictedApiInput;
import pps.core.prediction.service.data.predicted_api.PredictedApiOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.Collections;
import java.util.List;

/**
 * 预测API接口
 *
 * @author ZWT
 * @date 2024/04/29 16:01
 */
@XText("预测API接口")
@XService
public class PredictedApiService {

    /**
     * 一天预报
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link PredictedApiOutput}>
     */
    @XText("一天预报")
    @XApiPost(anonymous = true)
    public XListResult<PredictedApiOutput> oneDayForecast(XContext context, PredictedApiInput input) {
        String stationName = input.getStationName();
        if (CharSequenceUtil.isBlank(stationName)) {
            return XListResult.success(Collections.emptyList());
        }
        //取电站
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPhotovoltaicPlant(context, stationName);
        if (CollUtil.isEmpty(plantList)) {
            return XListResult.success(Collections.emptyList());
        }
        GetBasePhotovoltaicPlantCloudOutput plant = plantList.get(0);
        //查发电量
        PlantPredictedPowerShortTermMapper mapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
        List<PlantPredictedPowerShortTermEnt> list = mapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerShortTermEnt>()
                .eq(PlantPredictedPowerShortTermEnt::getPlantId, plant.getId())
                .between(PlantPredictedPowerShortTermEnt::getDataDate,
                        DateUtil.beginOfDay(DateUtil.date()).toString(),
                        DateUtil.beginOfDay(DateUtil.tomorrow()).toString())
                .orderByAsc(PlantPredictedPowerShortTermEnt::getDataDate)
        );
        //封装结果
        List<PredictedApiOutput> outputs = XCopyUtils.copyNewList(list, PredictedApiOutput.class);
        return XListResult.success(outputs);
    }

    /**
     * 十五天预报
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link PredictedApiOutput}>
     */
    @XText("十五天预报")
    @XApiPost(anonymous = true)
    public XListResult<PredictedApiOutput> fifteenDayForecast(XContext context, PredictedApiInput input) {
        String stationName = input.getStationName();
        if (CharSequenceUtil.isBlank(stationName)) {
            return XListResult.success(Collections.emptyList());
        }
        //取电站
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = this.getPhotovoltaicPlant(context, stationName);
        if (CollUtil.isEmpty(plantList)) {
            return XListResult.success(Collections.emptyList());
        }
        GetBasePhotovoltaicPlantCloudOutput plant = plantList.get(0);
        //查发电量
        DateTime begin = DateUtil.beginOfDay(DateUtil.date());
        PlantPredictedPowerMidTermMapper mapper = context.getBean(PlantPredictedPowerMidTermMapper.class);
        List<PlantPredictedPowerMidTermEnt> list = mapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerMidTermEnt>()
                .eq(PlantPredictedPowerMidTermEnt::getPlantId, plant.getId())
                .between(PlantPredictedPowerMidTermEnt::getDataDate,
                        begin.toString(),
                        DateUtil.offsetDay(begin, 15).toString())
                .orderByAsc(PlantPredictedPowerMidTermEnt::getDataDate)
        );
        //封装结果
        List<PredictedApiOutput> outputs = XCopyUtils.copyNewList(list, PredictedApiOutput.class);
        return XListResult.success(outputs);
    }

    /*--------------------------------- private --------------------------------*/

    /**
     * 获取光伏电站
     *
     * @param context     上下文
     * @param stationName 车站名称
     * @return {@link List}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    private List<GetBasePhotovoltaicPlantCloudOutput> getPhotovoltaicPlant(XContext context, String stationName) {
        BasePhotovoltaicPlantCloudService bean = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = bean.getBasePhotovoltaicPlantList(context, GetBasePhotovoltaicPlantCloudInput.builder()
                .stationName(stationName)
                .build());
        result.throwIfFail();
        return result.getResult();
    }
}
